package com.bergerkiller.bukkit.nolagg.lighting;

import org.bukkit.World;

/**
 * A single task the Lighting Service can handle
 */
public interface LightingTask {
	/**
	 * Gets the world this task is working on
	 * 
	 * @return task world
	 */
	public World getWorld();

	/**
	 * Checks whether a certain chunk is contained
	 * 
	 * @param chunkX
	 * @param chunkZ
	 * @return True if contained, False if not
	 */
	public boolean containsChunk(int chunkX, int chunkZ);

	/**
	 * Gets the amount of chunks this task is going to fix.
	 * This can be a wild estimate. While processing this amount should be
	 * updated as well.
	 * 
	 * @return estimated total chunk count
	 */
	public int getChunkCount();

	/**
	 * Processes this task (called from another thread!)
	 */
	public void process();

	/**
	 * Called from a synchronized task, ticking this task every tick
	 */
	public void syncTick();
}
